// const t = require('@babel/types');
const template = require('@babel/template');

function checkPath(options) {
  const filename = this.file.opts.filename;
  return options.include
    ? options.include.some(
        reg =>
          reg.test(filename) &&
          (options.exclude
            ? !options.exclude.some(reg => reg.test(filename))
            : true)
      )
    : true;
}

function createGraphql(path, options, query, name, rest = []) {
  const matchedGraphql =
    options.packages && options.packages.find(a => a.specifier === 'graphql');
  return t.callExpression(
    path.scope.bindings.graphql
      ? path.scope.bindings.graphql.identifier
      : t.toExpression(
          template.default(
            `require("${(matchedGraphql && matchedGraphql.source) ||
              '@apollo/react-hoc'}").graphql`
          )()
        ),
    [
      query,
      t.objectExpression([
        t.objectProperty(t.identifier('name'), name),
        ...rest
      ])
    ]
  );
}

module.exports = (options, t) => ({
  // ExportNamedDeclaration(path) {
  // if (
  //   path.node.declaration &&
  //   path.node.declaration.declarations &&
  //   (path.node.declaration.declarations[0].id.name === 'qwer' ||
  //     path.node.declaration.declarations[0].id.name === 'queryLocal' ||
  //     path.node.declaration.declarations[0].id.name === 'toggleNetwork')
  // ) {
  //   console.log(path.node.declaration.declarations[0].init);
  // }
  // },
  ExportDefaultDeclaration(path) {
    if (!checkPath.call(this, options)) return;

    let component;
    switch (path.node.declaration.type) {
      case 'ClassDeclaration':
      case 'FunctionDeclaration':
      case 'ArrowFunctionExpression':
        component = t.toExpression(path.node.declaration);
        break;
      case 'Identifier':
        component = path.node.declaration;
        break;
      default:
        break;
    }
    if (!component) return;
    let graphqls = [];
    path.container.forEach((node, i) => {
      if (t.isExportNamedDeclaration(node)) {
        if (node.declaration) {
          const query = node.declaration.declarations[0].init;
          // console.log(
          //   'id----',
          //   node.declaration.declarations[0].id.name,
          //   query.type
          // );
          let queryArgs, graphqlQuery, graphqlConfig;
          if (t.isArrayExpression(query)) {
            // with config
            [graphqlQuery, graphqlConfig] = query.elements;
          } else {
            // no config
            graphqlQuery = query;
          }

          if (
            (t.isTaggedTemplateExpression(graphqlQuery) &&
              (graphqlQuery.tag.name === 'gql' ||
                graphqlQuery.tag.name === 'graphql')) ||
            (t.isCallExpression(graphqlQuery) &&
              (graphqlQuery.callee.name === 'gql' ||
                graphqlQuery.callee.name === 'graphql'))
          ) {
            queryArgs = [graphqlQuery, graphqlConfig];
          }
          // console.log(query.arguments[0].callee);
          // !t.isArrayExpression(query) &&
          //   t.isTaggedTemplateExpression(query) &&
          //   console.log(query.tag.name);
          // component &&
          //   component.name &&
          //   component.name === 'User' &&
          //   console.log('===============', query.callee);
          if (queryArgs) {
            // matched graphql query
            const name = node.declaration.declarations[0].id.name;
            graphqls.push(
              createGraphql(
                path,
                options,
                queryArgs[0],
                t.stringLiteral(name),
                queryArgs[1] ? queryArgs[1].properties : []
              )
            );
            path.scope.bindings[name].references < 2 &&
              (path.container[i] = null);
          }
        } else if (node.source && /\.(gql|graphql)$/.test(node.source.value)) {
          node.specifiers.forEach(specifier => {
            graphqls.push(
              createGraphql(
                path,
                options,
                t.toExpression(
                  template.default(
                    `require('${node.source.value}').${specifier.exported.name}`
                  )()
                ),
                t.stringLiteral(specifier.exported.name)
              )
            );
          });
        }
      }
    });

    if (graphqls.length) {
      const matchedCompose =
        options.packages &&
        options.packages.find(a => a.specifier === 'compose');
      path.replaceWith(
        t.exportDefaultDeclaration(
          t.callExpression(
            graphqls.length > 1
              ? t.callExpression(
                  path.scope.bindings.compose
                    ? path.scope.bindings.compose.identifier
                    : t.toExpression(
                        template.default(
                          `require("${(matchedCompose &&
                            matchedCompose.source) ||
                            'lodash/flowRight'}")`
                        )()
                      ),
                  graphqls
                )
              : graphqls[0],
            [component]
          )
        )
      );
    }
  }
});
